@@ -28,14 +28,11 @@
 #include "nautilus-directory-private.h"
 #include "nautilus-enums.h"
 #include "nautilus-file-private.h"
-#include "nautilus-file-queue.h"
 #include "nautilus-file-utilities.h"
 #include "nautilus-global-preferences.h"
+#include "nautilus-hash-queue.h"
 #include "nautilus-metadata.h"
 #include "nautilus-scheme.h"
-#include "nautilus-search-directory-file.h"
-#include "nautilus-search-directory.h"
-#include "nautilus-starred-directory.h"
 #include "nautilus-vfs-directory.h"
 #include "nautilus-vfs-file.h"
 
@@ -124,32 +121,9 @@ real_new_file_from_filename (NautilusDirectory *directory,
                              const char        *filename,
                              gboolean           self_owned)
 {
-    NautilusFile *file;
-
-    g_assert (NAUTILUS_IS_DIRECTORY (directory));
-    g_assert (filename != NULL);
-    g_assert (filename[0] != '\0');
-
-    if (NAUTILUS_IS_SEARCH_DIRECTORY (directory))
-    {
-        if (self_owned)
-        {
-            file = NAUTILUS_FILE (g_object_new (NAUTILUS_TYPE_SEARCH_DIRECTORY_FILE, NULL));
-        }
-        else
-        {
-            /* This doesn't normally happen, unless the user somehow types in a uri
-             * that references a file like this. (See #349840) */
-            file = NAUTILUS_FILE (g_object_new (NAUTILUS_TYPE_VFS_FILE, NULL));
-        }
-    }
-    else
-    {
-        file = NAUTILUS_FILE (g_object_new (NAUTILUS_TYPE_VFS_FILE, NULL));
-    }
-    nautilus_file_set_directory (file, directory);
-
-    return file;
+    return NAUTILUS_FILE (g_object_new (NAUTILUS_TYPE_VFS_FILE,
+                                        "directory", directory,
+                                        NULL));
 }
 
 static gboolean
@@ -211,9 +185,9 @@ nautilus_directory_finalize (GObject *object)
     g_assert (directory->details->file_list == NULL);
     g_hash_table_destroy (directory->details->file_hash);
 
-    nautilus_file_queue_destroy (directory->details->high_priority_queue);
-    nautilus_file_queue_destroy (directory->details->low_priority_queue);
-    nautilus_file_queue_destroy (directory->details->extension_queue);
+    nautilus_hash_queue_destroy (directory->details->high_priority_queue);
+    nautilus_hash_queue_destroy (directory->details->low_priority_queue);
+    nautilus_hash_queue_destroy (directory->details->extension_queue);
     g_clear_list (&directory->details->files_changed_while_adding, g_object_unref);
     g_assert (directory->details->directory_load_in_progress == NULL);
     g_assert (directory->details->count_in_progress == NULL);
@@ -339,9 +313,9 @@ nautilus_directory_init (NautilusDirectory *directory)
     directory->details = nautilus_directory_get_instance_private (directory);
     directory->details->file_hash = g_hash_table_new_full (g_str_hash, g_str_equal,
                                                            g_free, NULL);
-    directory->details->high_priority_queue = nautilus_file_queue_new ();
-    directory->details->low_priority_queue = nautilus_file_queue_new ();
-    directory->details->extension_queue = nautilus_file_queue_new ();
+    directory->details->high_priority_queue = nautilus_hash_queue_new (g_direct_hash, g_direct_equal, g_object_ref, g_object_unref);
+    directory->details->low_priority_queue = nautilus_hash_queue_new (g_direct_hash, g_direct_equal, g_object_ref, g_object_unref);
+    directory->details->extension_queue = nautilus_hash_queue_new (g_direct_hash, g_direct_equal, g_object_ref, g_object_unref);
     directory->details->monitor_table = g_hash_table_new (NULL, NULL);
 }
 
@@ -687,6 +661,10 @@ nautilus_directory_new_file_from_filename (NautilusDirectory *directory,
                                            const char        *filename,
                                            gboolean           self_owned)
 {
+    g_assert (NAUTILUS_IS_DIRECTORY (directory));
+    g_assert (filename != NULL);
+    g_assert (filename[0] != '\0');
+
     return NAUTILUS_DIRECTORY_CLASS (G_OBJECT_GET_CLASS (directory))->new_file_from_filename (directory,
                                                                                               filename,
                                                                                               self_owned);
