@@ -3,6 +3,29 @@
 #include "nautilus-directory.h"
 #include "nautilus-global-preferences.h"
 
+/**
+ * NautilusViewModel:
+ *
+ * Internal structure goes like this:
+ *
+ * selection_model : GtkSelectionModel<GtkTreeListRow<NautilusViewItem>>
+ *  |
+ *  +-- sort_model : GtkSectionModel<GtkTreeListRow<NautilusViewItem>>
+ *       |
+ *       +-- tree_model : GtkTreeListModel<GtkTreeListRow<NautilusViewItem>>
+ *            |
+ *            +-- root_filter_model : GtkFilterListModel<NautilusViewItem>
+ *            |    |
+ *            |    +-- GListStore<NautilusViewItem>
+ *            |
+ *         (0...n) GtkFilterListModel<NautilusViewItem>  //subdirectories
+ *                 |
+ *                 +-- GListStore<NautilusViewItem>
+ *
+ * The overall model item type is GtkTreeListRow, but the :filter and :sorter
+ * properties are meant for internal models whose item type is NautilusViewItem.
+ */
+
 struct _NautilusViewModel
 {
     GObject parent_instance;
@@ -10,10 +33,12 @@ struct _NautilusViewModel
     GHashTable *map_files_to_model;
     GHashTable *directory_reverse_map;
 
+    GtkFilterListModel *root_filter_model;
     GtkTreeListModel *tree_model;
     GtkSortListModel *sort_model;
-    GtkMultiSelection *selection_model;
+    GtkSelectionModel *selection_model;
 
+    gboolean single_selection;
     gboolean expand_as_a_tree;
     GList *cut_files;
 };
@@ -27,7 +52,7 @@ get_directory_store (NautilusViewModel *self,
     store = g_hash_table_lookup (self->directory_reverse_map, directory);
     if (store == NULL)
     {
-        store = G_LIST_STORE (gtk_tree_list_model_get_model (self->tree_model));
+        store = G_LIST_STORE (gtk_filter_list_model_get_model (self->root_filter_model));
     }
 
     return store;
@@ -111,9 +136,8 @@ nautilus_view_model_is_selected (GtkSelectionModel *model,
                                  guint              position)
 {
     NautilusViewModel *self = NAUTILUS_VIEW_MODEL (model);
-    GtkSelectionModel *selection_model = GTK_SELECTION_MODEL (self->selection_model);
 
-    return gtk_selection_model_is_selected (selection_model, position);
+    return gtk_selection_model_is_selected (self->selection_model, position);
 }
 
 static GtkBitset *
@@ -122,9 +146,18 @@ nautilus_view_model_get_selection_in_range (GtkSelectionModel *model,
                                             guint              n_items)
 {
     NautilusViewModel *self = NAUTILUS_VIEW_MODEL (model);
-    GtkSelectionModel *selection_model = GTK_SELECTION_MODEL (self->selection_model);
 
-    return gtk_selection_model_get_selection_in_range (selection_model, pos, n_items);
+    return gtk_selection_model_get_selection_in_range (self->selection_model, pos, n_items);
+}
+
+static gboolean
+nautilus_view_model_select_item (GtkSelectionModel *model,
+                                 guint              position,
+                                 gboolean           unselect_rest)
+{
+    NautilusViewModel *self = NAUTILUS_VIEW_MODEL (model);
+
+    return gtk_selection_model_select_item (self->selection_model, position, unselect_rest);
 }
 
 static gboolean
@@ -133,12 +166,26 @@ nautilus_view_model_set_selection (GtkSelectionModel *model,
                                    GtkBitset         *mask)
 {
     NautilusViewModel *self = NAUTILUS_VIEW_MODEL (model);
-    GtkSelectionModel *selection_model = GTK_SELECTION_MODEL (self->selection_model);
-    gboolean res;
 
-    res = gtk_selection_model_set_selection (selection_model, selected, mask);
+    return gtk_selection_model_set_selection (self->selection_model, selected, mask);
+}
 
-    return res;
+
+static gboolean
+nautilus_view_model_unselect_item (GtkSelectionModel *model,
+                                   guint              position)
+{
+    NautilusViewModel *self = NAUTILUS_VIEW_MODEL (model);
+
+    return gtk_selection_model_unselect_item (self->selection_model, position);
+}
+
+static gboolean
+nautilus_view_model_unselect_all (GtkSelectionModel *model)
+{
+    NautilusViewModel *self = NAUTILUS_VIEW_MODEL (model);
+
+    return gtk_selection_model_unselect_all (self->selection_model);
 }
 
 
@@ -147,7 +194,10 @@ nautilus_view_model_selection_model_init (GtkSelectionModelInterface *iface)
 {
     iface->is_selected = nautilus_view_model_is_selected;
     iface->get_selection_in_range = nautilus_view_model_get_selection_in_range;
+    iface->select_item = nautilus_view_model_select_item;
     iface->set_selection = nautilus_view_model_set_selection;
+    iface->unselect_item = nautilus_view_model_unselect_item;
+    iface->unselect_all = nautilus_view_model_unselect_all;
 }
 
 G_DEFINE_TYPE_WITH_CODE (NautilusViewModel, nautilus_view_model, G_TYPE_OBJECT,
@@ -161,6 +211,8 @@ G_DEFINE_TYPE_WITH_CODE (NautilusViewModel, nautilus_view_model, G_TYPE_OBJECT,
 enum
 {
     PROP_0,
+    PROP_FILTER,
+    PROP_SINGLE_SELECTION,
     PROP_SORTER,
     N_PROPS
 };
@@ -194,6 +246,7 @@ dispose (GObject *object)
     }
 
     g_clear_object (&self->tree_model);
+    g_clear_object (&self->root_filter_model);
 
     G_OBJECT_CLASS (nautilus_view_model_parent_class)->dispose (object);
 }
@@ -221,6 +274,18 @@ get_property (GObject    *object,
 
     switch (prop_id)
     {
+        case PROP_FILTER:
+        {
+            g_value_set_object (value, nautilus_view_model_get_filter (self));
+        }
+        break;
+
+        case PROP_SINGLE_SELECTION:
+        {
+            g_value_set_boolean (value, nautilus_view_model_get_single_selection (self));
+        }
+        break;
+
         case PROP_SORTER:
         {
             g_value_set_object (value, nautilus_view_model_get_sorter (self));
@@ -244,6 +309,18 @@ set_property (GObject      *object,
 
     switch (prop_id)
     {
+        case PROP_FILTER:
+        {
+            nautilus_view_model_set_filter (self, g_value_get_object (value));
+        }
+        break;
+
+        case PROP_SINGLE_SELECTION:
+        {
+            self->single_selection = g_value_get_boolean (value);
+        }
+        break;
+
         case PROP_SORTER:
         {
             nautilus_view_model_set_sorter (self, g_value_get_object (value));
@@ -277,7 +354,13 @@ create_model_func (GObject           *item,
         g_hash_table_insert (self->directory_reverse_map, file, store);
     }
 
-    return g_object_ref (G_LIST_MODEL (store));
+    GtkFilterListModel *filter_model = gtk_filter_list_model_new (g_object_ref (G_LIST_MODEL (store)), NULL);
+
+    g_object_bind_property (self->root_filter_model, "filter",
+                            filter_model, "filter",
+                            G_BINDING_SYNC_CREATE);
+
+    return G_LIST_MODEL (filter_model);
 }
 
 static void
@@ -287,12 +370,28 @@ constructed (GObject *object)
 
     G_OBJECT_CLASS (nautilus_view_model_parent_class)->constructed (object);
 
-    self->tree_model = gtk_tree_list_model_new (G_LIST_MODEL (g_list_store_new (NAUTILUS_TYPE_VIEW_ITEM)),
+    self->root_filter_model = gtk_filter_list_model_new (G_LIST_MODEL (g_list_store_new (NAUTILUS_TYPE_VIEW_ITEM)), NULL);
+
+    self->tree_model = gtk_tree_list_model_new (g_object_ref (G_LIST_MODEL (self->root_filter_model)),
                                                 FALSE, FALSE,
                                                 (GtkTreeListModelCreateModelFunc) create_model_func,
                                                 self, NULL);
     self->sort_model = gtk_sort_list_model_new (g_object_ref (G_LIST_MODEL (self->tree_model)), NULL);
-    self->selection_model = gtk_multi_selection_new (g_object_ref (G_LIST_MODEL (self->sort_model)));
+
+    if (self->single_selection)
+    {
+        GtkSingleSelection *single = gtk_single_selection_new (NULL);
+
+        gtk_single_selection_set_autoselect (single, FALSE);
+        gtk_single_selection_set_can_unselect (single, TRUE);
+
+        gtk_single_selection_set_model (single, G_LIST_MODEL (self->sort_model));
+        self->selection_model = GTK_SELECTION_MODEL (single);
+    }
+    else
+    {
+        self->selection_model = GTK_SELECTION_MODEL (gtk_multi_selection_new (g_object_ref (G_LIST_MODEL (self->sort_model))));
+    }
 
     self->map_files_to_model = g_hash_table_new (NULL, NULL);
     self->directory_reverse_map = g_hash_table_new_full (NULL, NULL, NULL, g_object_unref);
@@ -316,9 +415,16 @@ nautilus_view_model_class_init (NautilusViewModelClass *klass)
     object_class->set_property = set_property;
     object_class->constructed = constructed;
 
+    properties[PROP_FILTER] =
+        g_param_spec_object ("filter", NULL, NULL,
+                             GTK_TYPE_FILTER,
+                             G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
+    properties[PROP_SINGLE_SELECTION] =
+        g_param_spec_boolean ("single-selection", NULL, NULL,
+                              FALSE,
+                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
     properties[PROP_SORTER] =
-        g_param_spec_object ("sorter",
-                             "", "",
+        g_param_spec_object ("sorter", NULL, NULL,
                              GTK_TYPE_SORTER,
                              G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
@@ -346,9 +452,39 @@ compare_data_func (gconstpointer a,
 }
 
 NautilusViewModel *
-nautilus_view_model_new (void)
+nautilus_view_model_new (gboolean single_selection)
 {
-    return g_object_new (NAUTILUS_TYPE_VIEW_MODEL, NULL);
+    return g_object_new (NAUTILUS_TYPE_VIEW_MODEL,
+                         "single-selection", single_selection,
+                         NULL);
+}
+
+GtkFilter *
+nautilus_view_model_get_filter (NautilusViewModel *self)
+{
+    return gtk_filter_list_model_get_filter (self->root_filter_model);
+}
+
+void
+nautilus_view_model_set_filter (NautilusViewModel *self,
+                                GtkFilter         *filter)
+{
+    if (self->root_filter_model == NULL ||
+        gtk_filter_list_model_get_filter (self->root_filter_model) == filter)
+    {
+        return;
+    }
+
+    gtk_filter_list_model_set_filter (self->root_filter_model, filter);
+    /* Subdirectory filter models are synchronized through bindings. */
+
+    g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_FILTER]);
+}
+
+gboolean
+nautilus_view_model_get_single_selection (NautilusViewModel *self)
+{
+    return self->single_selection;
 }
 
 GtkSorter *
@@ -375,6 +511,19 @@ nautilus_view_model_set_sorter (NautilusViewModel *self,
     g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_SORTER]);
 }
 
+/**
+ * Set the section sorter, effectively enabling sections.
+ *
+ * Unlike the regular sorter, which compares NautilusViewItem objects, this one
+ * compares the GtkTreeListRows objects which wrap the NautilusViewItem objects.
+ */
+void
+nautilus_view_model_set_section_sorter (NautilusViewModel *self,
+                                        GtkSorter         *section_sorter)
+{
+    gtk_sort_list_model_set_section_sorter (self->sort_model, GTK_SORTER (section_sorter));
+}
+
 void
 nautilus_view_model_sort (NautilusViewModel *self)
 {
@@ -473,7 +622,7 @@ nautilus_view_model_remove_items (NautilusViewModel *self,
 void
 nautilus_view_model_remove_all_items (NautilusViewModel *self)
 {
-    g_list_store_remove_all (G_LIST_STORE (gtk_tree_list_model_get_model (self->tree_model)));
+    g_list_store_remove_all (G_LIST_STORE (gtk_filter_list_model_get_model (self->root_filter_model)));
     g_hash_table_remove_all (self->map_files_to_model);
     g_hash_table_remove_all (self->directory_reverse_map);
 }
